from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.triggers.interval import IntervalTrigger
from apscheduler.schedulers.base import STATE_PAUSED, STATE_RUNNING
from controllers.message_controller import MessageController # Corrected import path
import logging
from threading import Lock
from config import FETCH_INTERVAL_MINUTES

logger = logging.getLogger(__name__)

# Global variables for scheduler management
scheduler_instance = [None, 0]  # [scheduler, empty_runs_counter]
fetch_lock = Lock()

def scheduled_fetch_messages():
    # Check if a fetch operation is already in progress
    if not fetch_lock.acquire(blocking=False):
        logger.info("Previous fetch operation still in progress, skipping this run")
        return
        
    try:
        logger.info("Starting scheduled fetch operation")
        
        # First process chats
        chats_count = MessageController.process_chats()
        logger.info(f"Processed {chats_count} chats")
        
        # Then process messages
        new_messages_count = MessageController.process_new_messages()
        logger.info(f"Processed {new_messages_count} new messages")
        
        # Get the scheduler instance
        scheduler = scheduler_instance[0]
        
        # Update scheduler state based on results
        if new_messages_count == 0 and chats_count == 0:
            scheduler_instance[1] += 1
            logger.info(f"No new data found. Empty runs: {scheduler_instance[1]}")
            
            if scheduler_instance[1] >= 3:
                logger.info("No new data for 3 consecutive runs. Pausing scheduler.")
                scheduler.pause()
        else:
            # Reset the counter if any data was found
            scheduler_instance[1] = 0
            
            # Resume scheduler if it was paused
            if scheduler.state == STATE_PAUSED:
                logger.info("New data found. Resuming scheduler.")
                scheduler.resume()
                
    except Exception as e:
        logger.error(f"Error in scheduled fetch: {str(e)}", exc_info=True)
    finally:
        fetch_lock.release()
        logger.info("Completed scheduled fetch operation")

def init_scheduler():
    scheduler = BackgroundScheduler()
    
    # Store scheduler instance and empty runs counter
    scheduler_instance[0] = scheduler
    scheduler_instance[1] = 0
    
    scheduler.add_job(
        func=scheduled_fetch_messages,
        trigger=IntervalTrigger(minutes=FETCH_INTERVAL_MINUTES),
        id='fetch_messages_job',
        name='Fetch new messages periodically',
        replace_existing=True,
        max_instances=1  # Ensure only one instance of the job can run at a time
    )
    scheduler.start()
    logger.info("Scheduled message fetching job started")
